@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2011, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,18 +18,9 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: strequal.c,v 1.33 2007-11-07 09:21:36 bagder Exp $
  ***************************************************************************/
 
-#ifndef _GNU_SOURCE
-/* glibc needs this to define the prototype for strcasestr */
-#define _GNU_SOURCE 1
-#endif
-
-#include "setup.h"
-
-#include <string.h>
-#include <ctype.h>
+#include "curl_setup.h"
 
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
@@ -37,12 +28,9 @@
 
 #include "strequal.h"
 
-#if defined(HAVE_STRCASECMP) && defined(__STRICT_ANSI__)
-/* this is for "-ansi -Wall -pedantic" to stop complaining! */
-extern int (strcasecmp)(const char *s1, const char *s2);
-extern int (strncasecmp)(const char *s1, const char *s2, size_t n);
-#endif
-
+/*
+ * @unittest: 1301
+ */
 int curl_strequal(const char *first, const char *second)
 {
 #if defined(HAVE_STRCASECMP)
@@ -63,13 +51,16 @@ int curl_strequal(const char *first, const char *second)
 #endif
 }
 
+/*
+ * @unittest: 1301
+ */
 int curl_strnequal(const char *first, const char *second, size_t max)
 {
-#if defined(HAVE_STRCASECMP)
+#if defined(HAVE_STRNCASECMP)
   return !strncasecmp(first, second, max);
-#elif defined(HAVE_STRCMPI)
+#elif defined(HAVE_STRNCMPI)
   return !strncmpi(first, second, max);
-#elif defined(HAVE_STRICMP)
+#elif defined(HAVE_STRNICMP)
   return !strnicmp(first, second, max);
 #else
   while(*first && *second && max) {
@@ -87,29 +78,6 @@ int curl_strnequal(const char *first, const char *second, size_t max)
 #endif
 }
 
-/*
- * Curl_strcasestr() finds the first occurrence of the substring needle in the
- * string haystack.  The terminating `\0' characters are not compared. The
- * matching is done CASE INSENSITIVE, which thus is the difference between
- * this and strstr().
- */
-char *Curl_strcasestr(const char *haystack, const char *needle)
-{
-#if defined(HAVE_STRCASESTR)
-  return strcasestr(haystack, needle);
-#else
-  size_t nlen = strlen(needle);
-  size_t hlen = strlen(haystack);
-
-  while(hlen-- >= nlen) {
-    if(curl_strnequal(haystack, needle, nlen))
-      return (char *)haystack;
-    haystack++;
-  }
-  return NULL;
-#endif
-}
-
 #ifndef HAVE_STRLCAT
 /*
  * The strlcat() function appends the NUL-terminated string src to the end
@@ -129,16 +97,19 @@ size_t Curl_strlcat(char *dst, const char *src, size_t siz)
   char *d = dst;
   const char *s = src;
   size_t n = siz;
-  size_t dlen;
+  union {
+    ssize_t sig;
+     size_t uns;
+  } dlen;
 
   /* Find the end of dst and adjust bytes left but don't go past end */
   while(n-- != 0 && *d != '\0')
     d++;
-  dlen = d - dst;
-  n = siz - dlen;
+  dlen.sig = d - dst;
+  n = siz - dlen.uns;
 
   if(n == 0)
-    return(dlen + strlen(s));
+    return(dlen.uns + strlen(s));
   while(*s != '\0') {
     if(n != 1) {
       *d++ = *s;
@@ -148,6 +119,6 @@ size_t Curl_strlcat(char *dst, const char *src, size_t siz)
   }
   *d = '\0';
 
-  return(dlen + (s - src));     /* count does not include NUL */
+  return(dlen.uns + (s - src));     /* count does not include NUL */
 }
 #endif
